CREATE TABLE IF NOT EXISTS sourceType (
        id            INTEGER PRIMARY KEY,
        name          VARCHAR(50)
        );

INSERT INTO sourceType VALUES (1, 'ldap');
INSERT INTO sourceType VALUES (2, 'ad');
INSERT INTO sourceType VALUES (3, 'text');


CREATE TABLE if NOT EXISTS attrList (
	id            INTEGER PRIMARY KEY,
	sourceType    INTEGER,
	name          VARCHAR(50),
	description   VARCHAR(100)
);

INSERT INTO attrList VALUES (null, 1, "sAMAccountName", "Logon Name (pre-Windows 1000)");
INSERT INTO attrList VALUES (null, 1, "userPrincipalName", "Logon Name");
INSERT INTO attrList VALUES (null, 1, "givenName", "First Name");
INSERT INTO attrList VALUES (null, 1, "middleName", "Middle Name");
INSERT INTO attrList VALUES (null, 1, "sn", "Last Name");
INSERT INTO attrList VALUES (null, 1, "company", "Company");
INSERT INTO attrList VALUES (null, 1, "department", "Department");
INSERT INTO attrList VALUES (null, 1, "co", "Country");
INSERT INTO attrList VALUES (null, 1, "l", "City");
INSERT INTO attrList VALUES (null, 1, "mail", "E-Mail Address");


CREATE TABLE IF NOT EXISTS source (
	id            INTEGER PRIMARY KEY AUTOINCREMENT,
	typeId        INTEGER,
	name          VARCHAR(50)
        );

CREATE TABLE IF NOT EXISTS sourceConfigElement (
	id              INTEGER PRIMARY KEY AUTOINCREMENT,
	typeId          INTEGER,
	name            VARCHAR(50),
	description     VARCHAR(50)
        );

INSERT INTO sourceConfigElement VALUES (null, 1, 'AuthID', '');
INSERT INTO sourceConfigElement VALUES (null, 1, 'Password', '');
INSERT INTO sourceConfigElement VALUES (null, 1, 'Server', '');
INSERT INTO sourceConfigElement VALUES (null, 1, 'Port', '');
INSERT INTO sourceConfigElement VALUES (null, 1, 'Schema', '');
INSERT INTO sourceConfigElement VALUES (null, 1, 'Path', '');
INSERT INTO sourceConfigElement VALUES (null, 3, 'File', '');
INSERT INTO sourceConfigElement VALUES (null, 3, 'Separator', '');


CREATE TABLE IF NOT EXISTS sourceConfig (
	id              INTEGER PRIMARY KEY AUTOINCREMENT,
	sourceId        INTEGER,
	sourceConfigElementId  INTEGER,
	value           VARCHAR(50)
	);

CREATE TABLE IF NOT EXISTS sourceField (
	id              INTEGER PRIMARY KEY AUTOINCREMENT,
	sourceId        INTEGER,
	name            VARCHAR(50),
	uid             TINYINT,
	history         TINYINT
        );

CREATE TABLE IF NOT EXISTS dataKey (
	id              INTEGER PRIMARY KEY AUTOINCREMENT,
	sourceFieldId   INTEGER,
	value           VARCHAR(25),
		UNIQUE (sourceFieldId, value)
	);



CREATE TABLE IF NOT EXISTS data (
	id              INTEGER PRIMARY KEY AUTOINCREMENT,
	dataKeyId       INTEGER,
	sourceFieldId   INTEGER,
	value           VARCHAR(25),
	updated         DATETIME
        );

CREATE INDEX [idData] ON [data] (
	[id] ASC 
	);

CREATE INDEX [dataKeyId] ON [data] (
	[dataKeyId] ASC,
	[sourceFieldId] ASC
	);


CREATE TABLE IF NOT EXISTS mapType (
	id		INTEGER PRIMARY KEY AUTOINCREMENT,
	name		VARCHAR(50)
);

INSERT INTO mapType VALUES (1, 'virtual');
INSERT INTO mapType VALUES (2, 'license-singel');
INSERT INTO mapType VALUES (3, 'license-multi');


CREATE TABLE IF NOT EXISTS mapConfig (
	id		INTEGER PRIMARY KEY AUTOINCREMENT,
	sourceId	INTEGER,
	file		VARCHAR(50),
	historic	TINYINT,
	mapTypeId	INTEGER
);


CREATE TABLE IF NOT EXISTS mapFields (
	id		INTEGER PRIMARY KEY AUTOINCREMENT,
	mapConfigId	INTEGER,
	sourceFieldId	INTEGER,
	fieldType	TINYINT
);


CREATE VIEW vConfig AS
  SELECT source.id AS sourceId, sourceConfigElement.name AS name, sourceConfig.value AS value FROM source
    INNER JOIN sourceConfigElement on source.typeId = sourceConfigElement.typeId
    INNER JOIN sourceConfig on sourceConfigElement.id = sourceConfig.sourceConfigElementId;


CREATE VIEW vField AS
  SELECT source.id AS sourceId, sourceField.id AS sourceFieldId, sourceField.name AS name, sourceField.uid AS uid, sourceField.history AS history FROM source
    INNER JOIN sourceField on source.id = sourceField.sourceId;


CREATE TRIGGER tDataInsert_updated AFTER INSERT ON data
BEGIN

  UPDATE data SET updated = DATETIME('NOW') WHERE rowid = new.rowid 
	AND new.updated IS null;

  DELETE FROM data WHERE
	data.sourceFieldId = new.sourceFieldId
    AND	data.dataKeyId = new.dataKeyId
    AND data.value = new.value
    AND (SELECT vField.history FROM vField WHERE vField.sourceFieldId = data.sourceFieldId) = 1
    AND data.id = (
	  SELECT
	    d2.id
	  FROM
	    data AS d2
	  WHERE
		d2.sourceFieldId = new.sourceFieldId
	    AND	d2.dataKeyId = new.dataKeyId
	  ORDER BY
	    d2.updated DESC
	  LIMIT 1,1
    );

  DELETE FROM
    data
  WHERE
      data.id != new.id
      AND (
	  SELECT
	    vField.history
	  FROM
	    vField
	  WHERE	    
	    vField.sourceFieldId = new.sourceFieldId
	) = 0
    AND data.sourceFieldId = new.sourceFieldId
    AND data.dataKeyId = new.dataKeyId;

END;


CREATE VIEW vData AS
  SELECT
    (SELECT dataKey.value FROM dataKey WHERE dataKey.id = data.dataKeyId) as id,
    (SELECT VField.sourceFieldId FROM vField WHERE vField.sourceFieldId = data.sourceFieldId) as sourceFieldId,
    (SELECT VField.name FROM vField WHERE vField.sourceFieldId = data.sourceFieldId) as name,
    data.value as value,
    data.updated as updated,
    (SELECT vField.history FROM vField WHERE vField.sourceFieldId = data.sourceFieldId) as history,
    (SELECT vField.uid FROM vField WHERE vField.sourceFieldId = data.sourceFieldId) as uid
  FROM
    data
  ORDER BY
    data.dataKeyId,
    uid DESC,
    data.sourceFieldId,
    data.updated;


CREATE VIEW vMap AS
  SELECT
    (SELECT name FROM source WHERE source.id = sourceId ) as source_name,
    file,
    historic,
    (SELECT name FROM mapType WHERE mapType.id = mapTypeId ) as map_type,
    (SELECT name FROM sourceField WHERE sourceField.id = mapFields.sourceFieldId) as source_field,
    fieldType
  FROM mapConfig
  LEFT OUTER JOIN mapFields on  mapConfig.id = mapFields.mapConfigId;
