#!/etc/opt/openit/perl
#
# $Id: sqlite.pl,v 1.1.2.1 2009/05/13 14:17:18 bolsen Exp $
#
# (C) Copyright 2008 by Open iT Norge AS, All rights reserved.
#
# This software may be used or copied only with the written permission of
# Open iT Norge AS or in accordance with the terms and conditions
# stipulated in the agreement under which this software has been supplied.
#
# This file is part of Open iT Resources
#
#HEAD##########################################################################
#
# @NAME:
#   sqlite
# @AUTHOR:
#   Borgar Olsen
# @CREATED:
#   Wed Dec  3 1:16:32 MET 2008
# @DESCRIPTION:
#   Simple script to run sql file on SQLite database
#
#/HEAD#########################################################################

#MAIN##########################################################################
#
# @HEADING Main Description goes here
#
# Remeber to document program. You can use @LINK="to other doc" and
# include images @IMG="my.image" in the document.
#
#/MAIN#########################################################################

use strict;

use DBI;

my ($db_path, $script) = @ARGV;

# Connect to SQLite database
my $dbh = DBI->connect( "dbi:SQLite:$db_path" ) || die "Cannot connect: $DBI::errstr";

&do_sql_statements( get_sql_statements($script) );

$dbh->disconnect;

exit 0;


sub get_sql_statements {
    my ($input) = @_;

    open(INP, $input);
    my @lines = <INP>;
    close(INP);

    # Remove comments etc.
    @lines = grep(! /^\s*--/, @lines);
    @lines = grep(! /^\s*$/, @lines);
    map { s/\n//g } @lines;
    map { s/\s+/ /g } @lines;
    map { s/^\s+//g } @lines;
    map { s/\s+$//g } @lines;

    # Join lines and resplit on ';'
    return &merge_sql_commands(@lines);
}


sub do_sql_statements {
    my @cmds = @_;

    foreach my $cmd (@cmds) {
	next if ($cmd =~ /^\s*$/); # Skip empty statements

	$dbh->do( $cmd );
    }
}

#
# Merge lines into single "complete" sql commands that can
# be executed by a "do".
#
sub merge_sql_commands {
    my @lines = @_;

    my @cmds;
    my @workset;

    my $in_block = 0;

    foreach my $line (@lines) {
	push @workset, $line;


	if ($line =~ /BEGIN/) {	 # Mark start of block
	    $in_block = 1;
	}
	elsif ( $line =~ /;/ ) { # Join if end of statment or block
	    if (! $in_block || $line =~ /END/ ) {
		$in_block = 0;
		push @cmds, join(" ", @workset);
		@workset = ();
	    }
	}
    }

    return @cmds;
}

